# The LearningOnline Network
# Simple Page Editor
#
# $Id: lonsimplepage.pm,v 1.26 2005/06/06 02:29:46 albertel Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#

package Apache::lonsimplepage;

use strict;
use Apache::Constants qw(:common);
use Apache::loncommon;
use Apache::lonnet;
use Apache::lontexconvert;
use Apache::lonfeedback;
use Apache::lonlocal;
use Apache::lonprintout;
use Apache::lonxml;

sub handler {
    my $r = shift;
    &Apache::loncommon::content_type($r,'text/html');
    $r->send_http_header;
    return OK if $r->header_only;
    my $target=$env{'form.grade_target'};
# ------------------------------------------------------------ Print the screen
    if ($target ne 'tex') {
	my $html=&Apache::lonxml::xmlbegin();
	$r->print(<<ENDDOCUMENT);
$html
<head>
<title>The LearningOnline Network with CAPA</title>
ENDDOCUMENT
    } else {
	$r->print(&Apache::lonprintout::print_latex_header($env{'form.latex_type'}));
    } 
    my (undef,undef,undef,undef,$marker)=split(/\//,$r->uri);
# Is this even in a course?
    unless ($env{'request.course.id'}) {
	if ($target ne 'tex') {
	    $r->print('</head><body>Not in a course</body></html>');
	    return OK;
	} else {
	    $r->print('\textbf{Not in a course}\end{document}');
	}
    }

    $marker=~s/\D//g;

    unless ($marker) {
	$r->print('<body>Invalid call</body>');
        return OK;
    }

    my $dom = $env{'course.'.$env{'request.course.id'}.'.domain'};
    my $crs = $env{'course.'.$env{'request.course.id'}.'.num'};

# --------------------------------------------------------- The syllabus fields
    my %syllabusfields=&Apache::lonlocal::texthash(
       'aaa_title'         => 'Page Title',
       'bbb_content'       => 'Content',
       'ccc_webreferences' => 'Web References');


# ------------------------------------------------------------ Get query string
    &Apache::loncommon::get_unprocessed_cgi
                        ($ENV{'QUERY_STRING'},['forcestudent','forceedit','register']);
# ----------------------------------------------------- Force menu registration
    my $addentries='';
    if ($env{'form.register'}) {
       $addentries=' onLoad="'.&Apache::lonmenu::loadevents().
	   '" onUnload="'.&Apache::lonmenu::unloadevents().'"';
       $r->print(&Apache::lonmenu::registerurl(1));
    }
# --------------------------------------------------------------- Force Student
    my $forcestudent='';
    if ($env{'form.forcestudent'} || $target eq 'tex' ) { $forcestudent='student'; };
     my $forceedit='';
     if ($env{'form.forceedit'}) { $forceedit='edit'; }


    my %syllabus=&Apache::lonnet::dump('smppage_'.$marker,$dom,$crs);
       
# --------------------------------------- There is such a user, get environment

    if ($target ne 'tex') {
	$r->print(&Apache::lonhtmlcommon::htmlareaheaders().
		  '</head>'.&Apache::loncommon::bodytag
		  ("Course Page",$forcestudent,$addentries,'',$dom,$env{'form.register'}));
    }

    my $allowed=&Apache::lonnet::allowed('mdc',$env{'request.course.id'});
    my $privileged=$allowed;
    if (($syllabus{'uploaded.lastmodified'}) && (!$forceedit)) {
	$forcestudent='student';
    }

    if ($forcestudent or $target eq 'tex') { $allowed=0; }
    
    if ($allowed) {
	$r->print('<p>'.
		  &Apache::loncommon::help_open_topic('Uploaded_Templates_TextBoxes','Help with filling in text boxes').'<br /><a href="'.$r->uri.'?forcestudent=1"><font size="+1">'.&mt('Show Student View').'</font></a>'.
		  &Apache::loncommon::help_open_topic('Uploaded_Templates_PublicView').'</p>');
    } elsif ($privileged and $target ne 'tex') {
	$r->print('<a href="'.$r->uri.'?forceedit=edit"><font size="+1">'.&mt('Edit').'</font></a>');
    } 
    if (($env{'form.uploaddoc.filename'} and $target ne 'tex') &&
	($env{'form.storeupl'}) && ($allowed)) {
	if ($env{'form.uploaddoc.filename'}=~/\.(gif|jpg|png|jpeg)$/i) {
	    if ($syllabus{'uploaded.photourl'}) {
		&Apache::lonnet::removeuploadedurl($syllabus{'uploaded.photourl'});
	    }
	    $syllabus{'uploaded.photourl'}=
		&Apache::lonnet::userfileupload('uploaddoc',1,'simplepage');
	}
	$syllabus{'uploaded.lastmodified'}=time;
	&Apache::lonnet::put('smppage_'.$marker,\%syllabus,$dom,$crs);
    }
    if (($allowed) && ($env{'form.storesyl'})) {
	foreach (keys %syllabusfields) {
	    my $field=$env{'form.'.$_};
	    chomp($field);
	    $field=~s/\s+$//s;
	    $field=~s/^\s+//s;
	    $field=~s/\<br\s*\/*\>$//s;
	    $field=&Apache::lonfeedback::clear_out_html($field,1);
	    $syllabus{$_}=$field;
	}
	$syllabus{'uploaded.lastmodified'}=time;
	&Apache::lonnet::put('smppage_'.$marker,\%syllabus,$dom,$crs);
    }

# ---------------------------------------------------------------- Get syllabus
    if (($syllabus{'uploaded.lastmodified'}) || ($allowed)) {
	if ($syllabus{'uploaded.photourl'}) {
	    &Apache::lonnet::allowuploaded('/adm/smppg',
					   $syllabus{'uploaded.photourl'});
	    
	    my $image='<img src="'.$syllabus{'uploaded.photourl'}.'"
                            align="right" />';
	    if ($target eq 'tex') {
		$image=&Apache::lonxml::xmlparse($r,'tex',$image);
	    }
	    $r->print($image);
	}
	if ($allowed) {
	    $r->print(
		      '<form method="post" enctype="multipart/form-data">'.
		      '<input type="hidden" name="forceedit" value="edit" />'.
		      '<h3>Upload a Photo</h3>'.
		      '<input type="file" name="uploaddoc" size="50">'.
		      '<input type="submit" name="storeupl" value="Upload">'.
		      '</form><form method="post">');
	}
	foreach (sort keys %syllabusfields) {
	    if (($syllabus{$_}) || ($allowed)) {
		my $message=$syllabus{$_};
		&Apache::lonfeedback::newline_to_br(\$message);
		$message
		    =~s/(https*\:\/\/[^\s]+)/\<a href=\"$1\"\>\<tt\>$1\<\/tt\>\<\/a\>/g;
		$message=&Apache::lontexconvert::msgtexconverted($message);
		if ($allowed) {
		    $message=&Apache::lonspeller::markeduptext($message);
		}
		unless ($_ eq 'aaa_title') {
		    if (($_ ne 'bbb_content') || ($allowed)) {
			if ($target ne 'tex') {
			    $r->print('<h3>'.$syllabusfields{$_}.'</h3>');
			} else {
			    my $safeinit;
			    $r->print(&Apache::lonxml::xmlparse($r,'tex','<h3>'.$syllabusfields{$_}.'</h3>'));
			}
		    }
		    if ($target ne 'tex') {
			$r->print('<blockquote>'.
				  $message.'</blockquote>');
		    } else {
			my $safeinit;
			$r->print(&Apache::lonxml::xmlparse($r,'tex',$message));
		    }
		    if ($allowed) {
			if ($target ne 'tex') {
			    $r->print('<br /><textarea cols="80" rows="24" name="'.$_.'" id="'.$_.'">'.
				      $syllabus{$_}.
				      '</textarea><input type="submit" name="storesyl" value="Store" />');
			} else {
			    my $safeinit;
			    $r->print(&Apache::lonxml::xmlparse($r,'tex',$syllabus{$_},$safeinit));
			}
		    }
		} else {
		    if ($target ne 'tex') {
			$r->print('<h1>'.$message.'</h1>');
		    } else {
			my $safeinit;
			$r->print(&Apache::lonxml::xmlparse($r,'tex','<h1>'.$message.'</h1>'));
		    }
		    if ($allowed) {
			if ($env{'form.grade_target'} ne 'tex') {
			    $r->print(
				      '<br />Title<br /><textarea cols="80" rows="2" name="'.$_.'">'.
				      $syllabus{$_}.
				      '</textarea><input type="submit" name="storesyl" value="Store" />');
			} else {
			    my $safeinit;
			    $r->print(&Apache::lonxml::xmlparse($r,'tex',$syllabus{$_},$safeinit));
			}
		    }
		}
	    }
	}
	if ($allowed && ($env{'form.grade_target'} ne 'tex')) {
	    $r->print(&Apache::lonhtmlcommon::htmlareaselectactive
		      ('bbb_content').'</form>');
	}
	if ($env{'form.grade_target'} ne 'tex') {$r->print('</p>');}
    } else {
	$r->print('<p>No page information provided.</p>');
    }
    if ($env{'form.grade_target'} ne 'tex') {
	$r->print('</body></html>');
    } else {
	$r->print('\end{document}');
    }
    return OK;
} 

1;
__END__
